from flask_wtf import FlaskForm
from wtforms import StringField, PasswordField, BooleanField, SubmitField, TextField, TextAreaField , SelectField
from wtforms.fields.html5 import SearchField, TelField, EmailField
from wtforms.validators import ValidationError, DataRequired, Email, EqualTo, Length, NoneOf, Regexp, IPAddress

from app.models.user import roles

class LoginForm(FlaskForm):
    username = StringField('Username', validators=[DataRequired()])
    password = PasswordField('Password', validators=[DataRequired()])
    remember_me = BooleanField('Remember Me')
    submit = SubmitField('Sign In') 


# last chance to change your mind form, used in several views
class OkCancelForm(FlaskForm):
    ok = SubmitField('Ok')
    cancel = SubmitField('Cancel')


# change user password
class PasswordChangeForm(FlaskForm):
    old_password = PasswordField('Current Password', validators=[DataRequired()])
    password = PasswordField('New Password', validators=[DataRequired(),Length(min=4, max=35,message="Password must be 4-35 characters"),Regexp('^[\w-]+$', message='Password can contain only alphanumeric characters (and _, -).')])
    password2 = PasswordField('Repeat New Password', validators=[DataRequired(), EqualTo('password', message='Passwords must match')])
    submit = SubmitField('Change')


# Admin - form to search for users in admin.html
class AdminForm(FlaskForm):
    search_term = SearchField('', validators=[Regexp('^[\w-]+$', message='Username parts can contain only alphanumeric characters (and _, -).')]) # no label here
    submit = SubmitField('Search')

#from app.model.user import roles
# add new user 
class AddUserForm(FlaskForm):
    invalidInput = ['root', 'bin', 'daemon', 'adm', 'lp', 'sync', 'shutdown', 'halt', 'mail', 'news', 'uucp', 'operator', 'operator', 'games', 'ftp', 'nobody', 'systemd-network', 'dbus', 'nscd', 'ldap', 'nslcd', 'saslauth', 'nginx', 'postfix', 'opendkim', 'dovecot', 'dovenull', 'clamupdate', 'tss', 'amavis', 'clamscan', 'prosody', 'unbound', 'apache', 'radicale', 'man', 'postmaster', 'cron', 'ftp', 'sshd', 'at', 'squid', 'xfs', 'cyrus', 'vpopmail', 'ntp', 'smmsp', 'guest', 'named']
    username = StringField('Username', validators=[DataRequired(),Length(min=2, max=25,message="Username must be 2-25 characters"),NoneOf(invalidInput, message="Select another username, this not available"),Regexp('^[\w-]+$', message='Username can contain only alphanumeric characters (and _, -).')])
    password = PasswordField('User Password', validators=[DataRequired(),Length(min=4, max=35,message="Password must be 4-35 characters"),Regexp('^[\w-]+$', message='Password can contain only alphanumeric characters (and _, -).')])
    password2 = PasswordField(
        'Repeat User Password', validators=[DataRequired(), EqualTo('password', message='Passwords must match')])
    gecos = StringField('Public name', validators=[DataRequired()])
    email = EmailField('Email address', validators=[Email()])
    phone = TelField('Phone number')
    IM = StringField('IM')
    isLocked = BooleanField('Locked out?')
    role = SelectField('Select Role', choices=[(role, role.capitalize()) for role in roles])
    submit = SubmitField('OK')


#from app.model.user import roles
# user_info or modify OTHER user data or status
class UserInfoForm(FlaskForm):
    gecos = StringField('Public name', validators=[DataRequired()])
    email = EmailField('Email address', validators=[Email()])
    phone = TelField('Phone number')
    IM = StringField('IM')
    # choices are later modified by app
    role = SelectField('Select Role', choices=[(role, role.capitalize()) for role in roles])
    isLocked = BooleanField('Is user locked out?')
    submit = SubmitField('OK')

# user_info or modify SELF user data or status
class SelfUserInfoForm(FlaskForm):
    gecos = StringField('Public name', validators=[DataRequired()])
    email = EmailField('Email address', validators=[Email()])
    phone = TelField('Phone number')
    IM = StringField('IM')
    submit = SubmitField('OK')


# search
class SearchForm(FlaskForm):
    expression = StringField("search for",validators=[DataRequired("Please enter search keywords")])
    submit = SubmitField('Search')


