from flask import request, jsonify, make_response, json
from flask import Flask, jsonify, request 
from flask import Blueprint, current_app

from flask import Flask, render_template, request, redirect, url_for, send_from_directory, flash, session
from flask_login import login_user, login_required, logout_user, current_user

from config import Config

import os
import logging
import json

from app.extensions import users as usrs
from app.extensions import Users, User, hash_me
from .forms import LoginForm, PasswordChangeForm, AddUserForm, AdminForm,  SearchForm, UserInfoForm, SelfUserInfoForm

bp = Blueprint('web',__name__, url_prefix='/web')

from functools import wraps
def admin_required(f):
    @login_required
    @wraps(f)
    def wrap(*args, **kwargs):
        my_username = current_user.get_id()
        if (usrs.get_my_role(my_username) == 'admin' or usrs.get_my_role(my_username) == 'owner'):
            return f(*args, **kwargs)
        else:
            flash('You must have administrator privileges to access this page.', 'error')
            logout()
            return redirect(url_for('web.index'))
    return wrap

def owner_required(f):
    @login_required
    @wraps(f)
    def wrap(*args, **kwargs):
        my_username = current_user.get_id()
        if (usrs.get_my_role(my_username) == 'owner'):
            return f(*args, **kwargs)
        else:
            flash('You must have owner privileges to access this page.', 'error')
            logout()
            return redirect(url_for('web.index'))
    return wrap



# Define a 404 error handler for the Blueprint
@bp.app_errorhandler(404)
def not_found_error(error):
    return render_template('404.html'), 404


# Define a 500 error handler for the Blueprint
@bp.app_errorhandler(500)
def not_found_error(error):
    return render_template('500.html'), 500

# SETTINGS +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''
Contains options for user settings
Password change, Mode
'''
@bp.route('/settings', methods=['GET', 'POST'])
@login_required
def settings():
    a_user = usrs.get_user(current_user.get_id()) # get me my user info
    form = SelfUserInfoForm(data=a_user.to_dict()) # generate zucking generic object for data edit
    if form.validate_on_submit():
        new_user_data=form.data # get all from the form
        new_user_data['username']=current_user.get_id() # add userid since it is missing from the form
        logging.info('my new_user_data: '+json.dumps(new_user_data))
        if(usrs.update_user(new_user_data)): 
            if usrs.chk_user_data():
                usrs.save() # change done and consistent, can save now
                flash('Your user data updated')
                return redirect(url_for('web.index'))
            else:
                flash('Your user data not changed')
                return redirect(url_for('web.settings'))                
        else:
            flash('Your user data not changed')
            return redirect(url_for('web.settings'))
    return render_template('settings.html', title='Settings', domain_name=current_app.config.get('WEB_DOMAIN_TLD','default-domain.com'), form=form)

@bp.route('/password_change', methods=['GET', 'POST'])
@login_required
def password_change():
    form = PasswordChangeForm()
    if form.validate_on_submit():
        # here hash password into crypted form from the form and compare
        hashed_form_old_password_data = hash_me(form.old_password.data)
        hashed_form_password_data = hash_me(form.password.data)
        if changePassword(hashed_form_old_password_data, hashed_form_password_data):
            flash('Password sucessfuly changed')
            return redirect(url_for('web.index'))
        else:
            flash('Invalid old password')
            return redirect(url_for('web.password_change')) 
    return render_template('password_change.html', title='Password change', domain_name=current_app.config.get('WEB_DOMAIN_TLD','default-domain.com'), form=form)

# sucesfully change self psswd
def changePassword(pw, newpw):
    my_username = current_user.get_id()
    # first chk if user knows his current psswd
    if not usrs.is_password(my_username,pw):
        # failed login, send him back to login page
        logging.warning('Invalid old password')
        return False
    # he does know, psswd valid, change psswd to new one
    usrs.change_password(my_username, pw, newpw)
    # and make persistent
    if usrs.chk_user_data():
        usrs.save()
        return True
    else:
        print("chk_admin_data consistency failure, admin data not saved")
    return False


# ADMIN +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''
User administration list panel for publisher or owner ONLY
owner can edit all
publisher can edit editors
'''
@bp.route('/admin', methods=['GET', 'POST'])
@login_required
@admin_required
def admin():
    l_users = []
    logging.info('users in admin: '+usrs.to_json() )
    # whats my role
    if(current_user.role=='admin'):
        # filter others and editors only
        l_users = [user for user in usrs.users if (user.username != current_user.get_id() or user.role == 'user')] # get self and users
    else:
        # filter all others except self
        logging.info('users in admin filter: '+usrs.to_json() )
        l_users = [user for user in usrs.users if (user.username != current_user.get_id())] # get self and editors
    search_term = None
    form=AdminForm()
    if form.validate_on_submit():
        search_term = form.search_term.data # search_term changed, make sure its ASCII
        # get me list of all users, according to search_term 
        l_users = usrs.search_users( search_term, current_user.get_id() ) 
    return render_template('admin.html', title='Admin', domain_name=current_app.config.get('WEB_DOMAIN_TLD','default-domain.com'),  users=l_users, form=form)

'''
Single user administration panel
'''
@bp.route('/user_info?userid=<userid>', methods=['GET', 'POST'])
@login_required
@admin_required
def user_info(userid):
    if(userid is None or userid == ''):
        return redirect(url_for('web.admin')) 
    a_user = usrs.get_user(userid) # get me the user info
    if(a_user==None): # should never happen
        return redirect(url_for('web.admin')) 
    if(current_user.get_id()!=userid): # not editing self, this is other user
        form = UserInfoForm( data=a_user.to_dict() ) # generate generic object for data edit
        form.role.choices=[(role, role.capitalize()) for role in usrs.possible_roles(current_user.get_id())] # reduce choices
    else: # editing self, not being able to edit change own admin status or Lock out self
        form = UserInfoForm( data=a_user.to_dict() ) # generate generic object for data edit
        form.role.choices=[(role, role.capitalize()) for role in usrs.possible_roles(current_user.get_id())] # reduce choices
    if form.validate_on_submit():
        if(current_user.get_id()!=userid): # not editing self
            new_user_data=form.data # get all from the form
            new_user_data['username']=userid # add userid since it is missing from the form
            logging.info('new_user_data: '+json.dumps(new_user_data))
            if(usrs.update_user(new_user_data)):
                usrs.save() # change done and consistent, can save now
                flash('User '+ userid +' data updated')
            else:
                flash('User '+ userid +' data not changed')
        else:
            new_user_data=form.data # get all from the form
            new_user_data['username']=userid # add userid since it is missing from the form
            logging.info('new_user_data: '+json.dumps(new_user_data))
            if(usrs.update_user(new_user_data)): 
                usrs.save() # change done and consistent, can save now
                flash('Your user data updated')
            else:
                flash('Your user data not changed')
        return redirect(url_for('web.admin')) 
    return render_template('user_info.html', title='User info', domain_name=current_app.config.get('WEB_DOMAIN_TLD','default-domain.com'), user=a_user, form=form)



'''
Adding new User panel
'''
@bp.route('/add_user', methods=['GET', 'POST'])
@login_required
@admin_required
def add_user():
    form = AddUserForm()
    form.role.choices=[(role, role.capitalize()) for role in usrs.possible_roles(current_user.get_id())] # reduce choices
    logging.warning('AddUserForm presented')
    if form.validate_on_submit():
        if len(usrs.users) >= current_app.config['MAX_USERS']:
            flash('Maximum number of users reached')
            return redirect(url_for('web.admin')) # dont get to same form, return to admin panel
        # first check if the new username already exist in LDAP
        if usrs.is_user( form.username.data ):
            flash('Existing username - use another usename')
        else:
            # prepare arguments
            new_user_data=form.data # get all from the form
            # here hash password into crypted form and replace
            new_user_data['password'] = hash_me( new_user_data['password'])
            # new
            # build new_user from new_user_data
            new_user = User.from_dict(new_user_data)
            usrs.users.append(new_user)
            if(usrs.chk_user_data()): 
                usrs.save() # change done and consistent, can save now
                flash('New user '+ form.username.data +' created')
                return redirect(url_for('web.admin')) # dont get to same form, return to admin panel
            else:
                flash('Invalid user data '+ form.username.data +' data not saved')
            
    return render_template('add_user.html', title='Add new user', domain_name=current_app.config.get('WEB_DOMAIN_TLD','default-domain.com'), form=form)




@bp.route('/login', methods=['GET', 'POST'])
def login():
    if current_user.is_authenticated:
        return redirect(url_for('web.index'))
    form = LoginForm()
    #logging.warning('LoginForm presented %s', session)
    if form.validate_on_submit():
        username = form.username.data
        password = hash_me(form.password.data) # hash password immediatelly
        #password = form.password.data
        # Check if the username and password match
        logging.warning('usrs %s ',usrs.to_dict())
        u = usrs.get_user(username)
        logging.warning('user %s ',u.get_id())
        if (u != None):
            if u.password == password:
                # create Flask User object
                #user = User(username,isAdmin, isOwner)
                user = User(username, u.gecos, u.role, u.password, u.isLocked, u.email, u.phone, u.IM)
                login_user(user)
                flash('Logged in successfully. Welcome '+user.get_id())
                return redirect(url_for('web.index'))
            flash('Invalid username or password')
        else:
            flash('Invalid username or password')
    return render_template('login.html', title='Sign In', domain_name=current_app.config.get('WEB_DOMAIN_TLD','default-domain.com'), form=form) # if login form did NOT validate_on_submit, return to the form


@bp.route('/logout')
@login_required
def logout():
    logout_user()
    return redirect(url_for('web.index'))

@bp.route('/favicon.ico')
def favicon():
    #return send_from_directory(app.config['STATIC_FOLDER'], 'favicon.ico',mimetype='image/vnd.microsoft.icon')
    return send_from_directory(current_app.config['STATIC_FOLDER'], 'favicon.png')


# if current_user.is_authenticated:
# this view should be like landing page on YT , and current should move to view point
@bp.route('/')
@bp.route('/index')
@login_required
def index():
    return render_template('index.html', anon=not current_user.is_authenticated, domain_name=current_app.config.get('WEB_DOMAIN_TLD','default-domain.com'))



# Route for toggling the mode
@bp.route('/toggle_mode')
def toggle_mode():
    # Toggle mode between 'light' and 'dark'
    session['mode'] = 'dark' if session.get('mode') == 'light' else 'light'
    logging.info('from tm session: %s', session)
    return '', 204





if __name__ == '__main__':
    app.run(debug=True)

    
 
