""" Sample infrastructure  object template """

import json

class Infra:
    """
    Sample infrastructure class

    :param id: this is a first param
    :param description: this is a second param
    :returns: self
    :raises keyError: raises an exception"""
    def __init__(self, id_ = None, description = None, attribute_1=None, attribute_2 = None,
                    boxes=[], metrics_boxes=[]):
        self.id_ = id_ # unique id of the infra
        self.description = description
        self.attribute_1 = attribute_1
        self.attribute_2 = attribute_2
        self.boxes = boxes # list Boxes
        self.metrics_boxes = metrics_boxes # list of other Boxes
        self.is_paused = None

    def set_attribute_1(self, attribute_1):
        """
        Sample seter

        :param attribute_1: this is a first param
        :returns: None"""
        self.attribute_1 = attribute_1

    def get_attribute_2(self):
        """ attribute_1 geter """
        return self.attribute_2

    def start(self): # start function
        self.is_paused = False
        return True

    def stop(self): # stop function
        result = True
        return result

    def pause(self): 
        self.is_paused = True
        return self.is_paused

    def unpause(self): 
        self.is_paused = False
        return True


    def to_dict(self):
        """ conversion to dictionary """
        result = {
            'id': self.id_,
            'description': self.description,
            'attribute_2': self.attribute_2,
            'boxes': [box.to_dict() for box in self.boxes],
            'metrics_boxes': [metrics_box.to_dict() for metrics_box in self.metrics_boxes],
            'is_paused': self.is_paused 
        }
        if self.attribute_1 is not None:
            result['attribute_1'] = self.attribute_1.to_dict()
        else:
            result['attribute_1'] = None
        return result

    def to_json(self):
        """ conversion to JSON"""
        return json.dumps(self.to_dict(), indent=2)

    def save_to_file(self, filename):
        """ save JSON to filename file"""
        with open(filename, 'w') as f_:
            json.dump(self.to_dict(), f_, indent=2)

    @staticmethod
    def from_dict(data):
        """ create from dictionary"""
        infra = Infra(id_='will_be_changed')
        infra.id_ = data.get('id')
        infra.description = data.get('description')
        infra.attribute_2 = data.get('attribute_2')
        infra.attribute_1 = data.get('attribute_1')
        infra.boxes = list(data.get('boxes')) # [box_data for box_data in data.get('boxes')]
        infra.metrics_boxes = list(data.get('metrics_boxes')) # [metrics_box_data for metrics_box_data in data.get('metrics_boxes')]
        infra.is_paused = data.get('is_paused')
        return infra

    @staticmethod
    def from_json(s_):
        """ create from JSON string"""
        data = json.loads(s_)
        return Infra.from_dict(data)

    @staticmethod
    def load_from_file(filename):
        """ create from filename file"""
        with open(filename, 'r') as f_:
            data = json.load(f_)
        return Infra.from_dict(data)


# note plural = a named list of Infra, used for storage manipulation
class Infras:
    """ a named list of Infra, used for storage manipulation"""
    def __init__(self, id_=None, description=None, filename=None):
        self.id_ = id_
        self.description = description
        self.data_list = []
        self.filename=filename

    def add(self, infra):
        """ adding new item """
        if infra is None:
            raise ValueError('Adding None infra')
        if ((infra.id_ is None) or (infra.id_) == ''):
            raise ValueError('Adding infra with no id_')
        if self.get(infra.id_) is None: # does not exist in infras already
            self.data_list.append(infra) # append
            self.save_to_file() # and save_to_file
            return True
        return False

    def remove(self, infra):
        """remove an item """
        if infra in self.data_list:
            self.data_list.remove(infra)
            self.save_to_file() # and save_to_file
            return True
        return False

    def get(self, infra_id): # find by id of the first infra, in the list
        """get item by id"""
        for x_ in self.data_list:
            if x_.id_ == infra_id:
                return x_
        return None

    def ls(self): # return a list of infra names
        """list items"""
        return [x_.id_ for x_ in self.data_list]

    def to_dict(self, include_subtree=True):
        """transfer to dictionary"""
        data = {
            'id': self.id_,
            'description': self.description,
            'filename': self.filename,
        }
        if include_subtree:
            data['data_list'] = [c.to_dict() for c in self.data_list if c is not None]
        return data

    def to_json(self, include_subtree=True):
        """transfer to JSON"""
        return json.dumps(self.to_dict(include_subtree), indent=2)

    def save_to_file(self, filename=None):
        """save to file"""
        fname = None
        if self.filename is not None:
            fname = self.filename
        elif filename is not None:
            fname = filename
        else:
            fname  = 'csystems.json'
        with open(fname, 'w') as f_:
            json.dump(self.to_dict(), f_, indent=2)

    @staticmethod
    def from_dict(data):
        """read from dictionry"""
        infras = Infras(
            id_=data['id'],
            description=data['description'],
            filename=data['filename'],
        )
        for x_ in data.get('data_list', []):
            a_infra = Infra.from_dict(x_)
            infras.add(a_infra)
        return infras

    @staticmethod
    def from_json(s):
        """create from JSON"""
        data = json.loads(s)
        return Infras.from_dict(data)

    @staticmethod
    def load_from_file(filename):
        """create from file"""
        with open(filename, 'r') as f_:
            data = json.load(f_)
        return Infras.from_dict(data)



def initialize(directory_path='data/'):
    """initializing function """
    import os
    def is_directory_empty(directory_path):
        """Check if a directory is empty."""
        # List the directory contents
        contents = os.listdir(directory_path)
        # If the directory contents are empty, return True, otherwise return False
        return len(contents) == 0

    # if there are data files just return, else create files
    if not is_directory_empty(directory_path):
        return

    infra = Infra(id_ = 'testinfra', description='test infra realistic ')

    # wrapping up =============================================================
    infras = Infras( id_='infras',
                    description='srvr part holding infras',
                    filename=directory_path+'infras.json')
    infras.add(infra)
    infras.save_to_file()

