import pytest
import requests

BASE_URL = "http://localhost:5000/auth"

@pytest.fixture
def setup_tree():
    # Step 1: Generate a token for bob
    token_response = requests.post(
        f"{BASE_URL}/token",
        headers={"Content-Type": "application/json"},
        json={"username": "bob", "password": "password2"}
    )
    # Assert token generation is successful
    assert token_response.status_code == 200, f"Unexpected status code: {token_response.status_code}"
    token_data = token_response.json()
    assert "token" in token_data, "Token not found in response"
    token = token_data["token"]
    assert token.startswith("eyJ"), "Token does not appear to be a valid JWT"
    return token


def test_get_token(setup_tree):
    token = setup_tree
    assert token.startswith("eyJ"), "Token does not appear to be a valid JWT"

def test_token_validation(setup_tree):
    token = setup_tree

    # Step 2: Validate the token
    test_response = requests.get(
        f"{BASE_URL}/test",
        headers={"Authorization": f"Bearer {token}"}
    )

    # Assert token validation is successful
    assert test_response.status_code == 200, f"Unexpected status code: {test_response.status_code}"
    test_data = test_response.json()
    assert "message" in test_data, "Message not found in response"
    assert test_data["message"] == "Access granted for user bob.", "Unexpected validation message"



def test_token_generation_and_validation():
    # Step 1: Generate a token
    token_response = requests.post(
        f"{BASE_URL}/token",
        headers={"Content-Type": "application/json"},
        json={"username": "bob", "password": "password2"}
    )

    # Assert token generation is successful
    assert token_response.status_code == 200, f"Unexpected status code: {token_response.status_code}"
    token_data = token_response.json()
    assert "token" in token_data, "Token not found in response"

    token = token_data["token"]
    assert token.startswith("eyJ"), "Token does not appear to be a valid JWT"

    # Step 2: Validate the token
    test_response = requests.get(
        f"{BASE_URL}/test",
        headers={"Authorization": f"Bearer {token}"}
    )

    # Assert token validation is successful
    assert test_response.status_code == 200, f"Unexpected status code: {test_response.status_code}"
    test_data = test_response.json()
    assert "message" in test_data, "Message not found in response"
    assert test_data["message"] == "Access granted for user bob.", "Unexpected validation message"
